@@ -20,7 +20,7 @@
 #define B_FIND_FOLLOW_SYMLINKS (1 << 0)
 #define B_FIND_CALLBACK(c)     ((b_find_callback)c)
 
-typedef int (*b_find_callback)(b_builder *builder, b_string *path, b_string *member_name, struct stat *st);
+typedef int (*b_find_callback)(b_builder *builder, b_string *path, b_string *member_name, struct stat *st, int fd);
 
 int b_find(b_builder *builder, b_string *path, b_string *member_name, b_find_callback callback, int flags);
 
